#include <iostream.h>
#include <fstream.h> 
#include <stdlib.h>  
#include <math.h>
#include <string>

bool checkValidSolution(int **grid, int sqLen, int grLen, int first);
void swapCellCols(int** grid, int grLen, int c1, int c2);
void swapCellRows(int** grid, int grLen, int r1, int r2);
void swapSquareRows(int** grid, int grLen, int sqLen, int r1, int r2);
void swapSquareCols(int** grid, int grLen, int sqLen, int c1, int c2);



int main(int argc, char **argv)
{
	if(argc <=1) {
		cout<<"\nSudoku Random Instance Generator\n";
		cout<<"USAGE: inFileName outFileName removeRate seed \n"<<endl;
		cout<<"-- the file specified by 'inFileName' must contain a valid and complete sudoku\nsolution in the specified format (see the supplied examples)\n"; 
		cout<<"-- removalRate must be between 0 and 1 inclusive)"<<endl;
		cout<<"-- seed must be a positive integer\n"<<endl;
		exit(1);
	}

	srand(atoi(argv[4]));
	double rrate = atof(argv[3]);

	int sqLen,r,c,first,grLen,i,temp;

	ifstream inStream;
	inStream.open(argv[1]);
	if(inStream.fail())	{
		cout << "There was an error opening the input solution file. Sorry.\n";
		exit(1);
	}

	//Read the file into an array
	inStream>>sqLen;
	inStream>>first;
	grLen = sqLen*sqLen;
	int **grid = new int* [grLen];
	for(i=0;i<grLen;i++) grid[i] = new int [grLen];

	//Read in the input and check that the thing is full, if not, exit
	for(r=0;r<grLen;r++){
		for(c=0;c<grLen;c++){
			inStream>>temp;
			grid[r][c]=temp;
			if(temp == -1){
				cout<<"Error: This is not a complete instance\n";
				exit(1);
			}
		}
	}
	inStream.close();

	//The grid is full. Check that it is is a valid solution
	if(!checkValidSolution(grid,sqLen,grLen,first)){
		cout<<"Error. The initial grid is not valid"<<endl;
		exit(1);
	}
	//else cout<<"The initial grid is valid"<<endl;		

	//IF WE ARE HERE, WE ARE READY TO SHUFFLE THE GRID.
	//1) Shuffle the cell columns
	int sq, j;
	//Go through each column of squares
	//cout<<"\n\nShuffling cell columns........."<<endl;
	for(sq=0;sq<sqLen;sq++){
		//cout<<"SQUARE "<<sq<<endl;
		//Randomly shuffle the columns within these squares
		for(i=sqLen-1; i>=0; i--){
		//for(i=numItems-1; i>1; i--){
			j = rand()%(i+1);
			//cout<<"Swapping columns "<<sq*sqLen+i<<" and "<<sq*sqLen+j<<endl;
			swapCellCols(grid,grLen,sq*sqLen+i,sq*sqLen+j);
		}
	}
	//2) Shuffle each row of each square
	//cout<<"\n\nShuffling cell rows........."<<endl;
	for(sq=0;sq<sqLen;sq++){
		//cout<<"SQUARE "<<sq<<endl;
		//Randomly shuffle the columns within these squares
		for(i=sqLen-1; i>=0; i--){
		//for(i=numItems-1; i>1; i--){
			j = rand()%(i+1);
			//cout<<"Swapping rows "<<sq*sqLen+i<<" and "<<sq*sqLen+j<<endl;
			swapCellRows(grid,grLen,sq*sqLen+i,sq*sqLen+j);
		}
	}
	//3)Shuffle rows of squares
	//cout<<"\n\nShuffling square columns........."<<endl;
	for(i=sqLen-1; i>=0; i--){
	//for(i=numItems-1; i>1; i--){
		j = rand()%(i+1);
		//cout<<"Swapping square columns "<<i<<" and "<<j<<endl;
		swapSquareCols(grid,grLen,sqLen,i,j);
	}

	//3)Shuffle columns of squares
	//cout<<"\n\nShuffling square rows........"<<endl;
	for(i=sqLen-1; i>=0; i--){
	//for(i=numItems-1; i>1; i--){
		j = rand()%(i+1);
		//cout<<"Swapping square rows "<<i<<" and "<<j<<endl;
		swapSquareRows(grid,grLen,sqLen,i,j);
	}

	if(!checkValidSolution(grid,sqLen,grLen,first)){
		cout<<"Error. The final grid is not valid"<<endl;
		exit(1);
	}

	//Now, go through each cell, and remove it with a rrate probability
	double x;
	for(r=0;r<grLen;r++){
		for(c=0;c<grLen;c++){
			x = rand()/double(RAND_MAX);
			if(x<=rrate) {
				//cout<<"Removing cell ["<<r<<","<<c<<"]\n";
				grid[r][c] = -1;
			}
		}
	}
	

	//want to output it to a file
	ofstream Os;
	Os.open(argv[2]);
	Os<<sqLen<<endl;
	Os<<first<<endl;
	for(r=0;r<grLen;r++){
		for(c=0;c<grLen;c++){
			Os<<grid[r][c]<<'\t';
		}
		Os<<endl;
	}
	Os.close();
	
	cout<<"The new instance has been written to the specified output file."<<endl; 

	return(0);
}

//-------------------------------------------------------------------------------------
bool checkValidSolution(int **grid, int sqLen, int grLen, int first)
{
	int r, c, j, count, r1, c1;
	bool valid = true;

	//Go through each row and check for duplicates
	for(r=0;r<grLen;r++){
		j=first;
		while(j<first+grLen){
			count=0;
			for(c=0;c<grLen;c++){
				if(grid[r][c]==j) count++;
			}
			if(count!=1){
				cout<<"Error. There are "<<count<<" occurences of "<<j<<" on row "<<r<<endl;
				valid = false;
			}
			j++;
		}
	}
	
	//Then columns
	for(c=0;c<grLen;c++){
		j=first;
		while(j<first+grLen){
			count=0;
			for(r=0;r<grLen;r++){
				if(grid[r][c]==j) count++;
			}
			if(count!=1){
				cout<<"Error. There are "<<count<<" occurences of "<<j<<" in column "<<c<<endl;
				valid = false;
			}
			j++;
		}
	}

	//Then Squares
	for(r=0;r<sqLen;r++){
		for(c=0;c<sqLen;c++){
			j=first;
			while(j<first+grLen){
				count=0;
				//This goes through square r c
				for(r1=r*sqLen; r1<(r+1)*sqLen;r1++){
					for(c1=c*sqLen; c1<(c+1)*sqLen;c1++){
						if(grid[r1][c1]==j) count++;
					}
				}
				if(count!=1){
					cout<<"Error. There are "<<count<<" occurences of "<<j<<" in square ["<<r<<","<<c<<"]\n";
					valid = false;
				}
				j++;
			}
		}
	}
	return(valid);
}

//-------------------------------------------------------------------------------------
void swapCellCols(int** grid, int grLen, int c1, int c2)
{
	int temp, r;
	for(r=0;r<grLen;r++){
		temp = grid[r][c1]; grid[r][c1] = grid[r][c2]; grid[r][c2] = temp;
	}
}

//-------------------------------------------------------------------------------------
void swapCellRows(int** grid, int grLen, int r1, int r2)
{
	int temp, c;
	for(c=0;c<grLen;c++){
		temp = grid[r1][c]; grid[r1][c] = grid[r2][c]; grid[r2][c] = temp;
	}
}

//-------------------------------------------------------------------------------------
void swapSquareRows(int** grid, int grLen, int sqLen, int r1, int r2)
{
	int i,c,temp;
	for(c=0;c<grLen;c++){
		for(i=0; i<sqLen; i++){
			temp = grid[r1*sqLen+i][c]; grid[r1*sqLen+i][c] = grid[r2*sqLen+i][c]; grid[r2*sqLen+i][c] = temp;
		}
	}
}

//-------------------------------------------------------------------------------------
void swapSquareCols(int** grid, int grLen, int sqLen, int c1, int c2)
{
	int i,r,temp;
	for(r=0;r<grLen;r++){
		for(i=0; i<sqLen; i++){
			temp = grid[r][c1*sqLen+i]; grid[r][c1*sqLen+i] = grid[r][c2*sqLen+i]; grid[r][c1*sqLen+i] = temp;
		}
	}
}
//-------------------------------------------------------------------------------------
